@@ -1,7 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from decimal import Decimal
-from functools import partial
 
 from trytond import backend
 from trytond.model import ModelView, fields
@@ -10,8 +9,9 @@ from trytond.pyson import Eval
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 
-from trytond.modules.account_product import MissingFunction
-from trytond.modules.product import price_digits, TemplateFunction
+from trytond.modules.account_product.product import (
+    account_used, template_property)
+from trytond.modules.product import price_digits
 
 __all__ = ['Category', 'CategoryAccount', 'Template', 'TemplateAccount',
     'Product', 'UpdateCostPriceAsk',
@@ -84,20 +84,6 @@ class Category:
                     | ~Eval('accounting', False)),
                 },
             depends=['account_parent', 'accounting']))
-    account_stock_used = MissingFunction(fields.Many2One('account.account',
-        'Account Stock Used'), 'missing_account', 'get_account')
-    account_stock_supplier_used = MissingFunction(fields.Many2One(
-            'account.account', 'Account Stock Supplier Used'),
-        'missing_account', 'get_account')
-    account_stock_customer_used = MissingFunction(fields.Many2One(
-            'account.account', 'Account Stock Customer Used'),
-        'missing_account', 'get_account')
-    account_stock_production_used = MissingFunction(fields.Many2One(
-            'account.account', 'Account Stock Production Used'),
-        'missing_account', 'get_account')
-    account_stock_lost_found_used = MissingFunction(fields.Many2One(
-        'account.account', 'Account Stock Lost and Found'),
-        'missing_account', 'get_account')
 
     @classmethod
     def multivalue_model(cls, field):
@@ -106,6 +92,31 @@ class Category:
             return pool.get('product.category.account')
         return super(Category, cls).multivalue_model(field)
 
+    @property
+    @account_used('account_stock')
+    def account_stock_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_supplier')
+    def account_stock_supplier_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_customer')
+    def account_stock_customer_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_production')
+    def account_stock_production_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_lost_found')
+    def account_stock_lost_found_used(self):
+        pass
+
 
 class CategoryAccount:
     __metaclass__ = PoolMeta
@@ -236,20 +247,6 @@ class Template:
                 }, help='This account will be used instead of the one defined '
             'on the category.',
             depends=['account_category', 'type']))
-    account_stock_used = MissingFunction(fields.Many2One('account.account',
-        'Account Stock Used'), 'missing_account', 'get_account')
-    account_stock_supplier_used = MissingFunction(fields.Many2One(
-            'account.account', 'Account Stock Supplier Used'),
-        'missing_account', 'get_account')
-    account_stock_customer_used = MissingFunction(fields.Many2One(
-            'account.account', 'Account Stock Customer Used'),
-        'missing_account', 'get_account')
-    account_stock_production_used = MissingFunction(fields.Many2One(
-            'account.account', 'Account Stock Production Used'),
-        'missing_account', 'get_account')
-    account_stock_lost_found_used = MissingFunction(fields.Many2One(
-            'account.account', 'Account Stock Lost and Found'),
-        'missing_account', 'get_account')
 
     @classmethod
     def __setup__(cls):
@@ -268,6 +265,31 @@ class Template:
             return pool.get('product.template.account')
         return super(Template, cls).multivalue_model(field)
 
+    @property
+    @account_used('account_stock')
+    def account_stock_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_supplier')
+    def account_stock_supplier_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_customer')
+    def account_stock_customer_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_production')
+    def account_stock_production_used(self):
+        pass
+
+    @property
+    @account_used('account_stock_lost_found')
+    def account_stock_lost_found_used(self):
+        pass
+
 
 class TemplateAccount:
     __metaclass__ = PoolMeta
@@ -333,22 +355,15 @@ class TemplateAccount:
 class Product:
     __metaclass__ = PoolMeta
     __name__ = 'product.product'
-    # Avoid raise of UserError from MissingFunction
-    account_stock_used = fields.Function(
-        fields.Many2One('account.account', 'Account Stock Used'),
-        'get_template')
-    account_stock_supplier_used = fields.Function(
-        fields.Many2One('account.account', 'Account Stock Supplier Used'),
-        'get_template')
-    account_stock_customer_used = fields.Function(
-        fields.Many2One('account.account', 'Account Stock Customer Used'),
-        'get_template')
-    account_stock_production_used = fields.Function(
-        fields.Many2One('account.account', 'Account Stock Production Used'),
-        'get_template')
-    account_stock_lost_found_used = fields.Function(
-        fields.Many2One('account.account', 'Account Stock Lost and Found'),
-        'get_template')
+    account_stock_used = template_property('account_stock_used')
+    account_stock_supplier_used = template_property(
+        'account_stock_supplier_used')
+    account_stock_customer_used = template_property(
+        'account_stock_customer_used')
+    account_stock_production_used = template_property(
+        'account_stock_production_used')
+    account_stock_lost_found_used = template_property(
+        'account_stock_lost_found_used')
 
 
 class UpdateCostPriceAsk(ModelView):
@@ -416,37 +431,22 @@ class UpdateCostPrice(Wizard):
     def default_ask_price(self, fields):
         pool = Pool()
         Product = pool.get('product.product')
-        Template = pool.get('product.template')
 
         context = Transaction().context
         default = {}
-        if ('product' in fields
-                and context['active_model'] == 'product.product'):
-            product = Product(context['active_id'])
-            template = product.template
-            default['product'] = product.id
-        elif ('template' in fields
-                and context['active_model'] == 'product.template'):
-            template = Template(context['active_id'])
-            product = template.products[0]
-            default['template'] = template.id
-        else:
-            return default
-        default['cost_price'] = getattr(product, 'recompute_cost_price_%s' %
-            product.cost_price_method)()
+        product = Product(context['active_id'])
+        default['product'] = product.id
+        default['cost_price'] = getattr(
+            product, 'recompute_cost_price_%s' % product.cost_price_method)()
         return default
 
     @staticmethod
     def get_product():
-        'Return the product or template instance'
+        'Return the product instance'
         pool = Pool()
         Product = pool.get('product.product')
-        ProductTemplate = pool.get('product.template')
         context = Transaction().context
-        if context['active_model'] == 'product.product':
-            return Product(context['active_id'])
-        else:
-            return ProductTemplate(context['active_id'])
+        return Product(context['active_id'])
 
     @classmethod
     def get_quantity(cls):
@@ -459,10 +459,7 @@ class UpdateCostPrice(Wizard):
         with Transaction().set_context(locations=[l.id for l in locations],
                 stock_date_end=stock_date_end):
             product = cls.get_product()
-            if hasattr(product.__class__, 'cost_price'):
-                return product.quantity
-            else:
-                return product.template.quantity
+            return product.quantity
 
     def transition_should_show_move(self):
         if self.get_quantity() != 0:
@@ -538,18 +535,6 @@ class UpdateCostPrice(Wizard):
         return 'update_price'
 
     def transition_update_price(self):
-        pool = Pool()
-        Product = pool.get('product.product')
-        ProductTemplate = pool.get('product.template')
-
-        if not isinstance(Product.cost_price, TemplateFunction):
-            write = partial(Product.write, [self.ask_price.product])
-        else:
-            if self.ask_price.template:
-                template = self.ask_price.template
-            else:
-                template = self.ask_price.product.template
-            write = partial(ProductTemplate.write, [template])
-
-        write({'cost_price': self.ask_price.cost_price})
+        self.ask_price.product.set_multivalue(
+            'cost_price', self.ask_price.cost_price)
         return 'end'
